/*
 * test_state_machine.c
 *
 * Created: 1/3/2020 9:28:34 AM
 *  Author: Taylor
 */ 
#include "mikrobridge.h"
#include "test_state_machine.h"

static state_t current_state = INIT;

ISR(PORTF_PORT_vect)
{
	/* Insert your PORTF interrupt handling code here */
	if(VPORTF_INTFLAGS & (1 << 6))
	{
		current_state += 1;
	}
	
	/* Clear interrupt flags */
	VPORTF_INTFLAGS = (1 << 6);
}

void run_state_machine() {
	// State machine that mirrors the VHDL testbench
	uint8_t read_result;
	uint8_t test_result;
	uint8_t spi_buffer[2];
	
	if(RST_STATE_get_level() == false)
	{
		current_state = INIT;
	}
	
	switch(current_state)
	{
		case INIT:
			// Reset error notification
			ERR_LED_set_level(false);
			GOOD_LED_set_level(false);
			RST_set_level(true);
			
			break;
		case VERIFY_REGS:
			// Reset error notification
			GOOD_LED_set_level(false);
			ERR_LED_set_level(false);
			test_result = 1;
			
			// Test power-on values of the config memory
			// This state will display an error after any of the locations have been modified during testing
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_SPI_SEL | MIKROBRIDGE_READ_MASK);
			test_result &= test_value(read_result, 0x00);
			
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_UART_SEL | MIKROBRIDGE_READ_MASK);
			test_result &= test_value(read_result, 0x00);
			
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_INT_SEL | MIKROBRIDGE_READ_MASK);
			test_result &= test_value(read_result, 0x00);
			
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_PWM_SEL | MIKROBRIDGE_READ_MASK);
			test_result &= test_value(read_result, 0x00);
			
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_AN_SEL | MIKROBRIDGE_READ_MASK);
			test_result &= test_value(read_result, 0x00);
			
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_RST_SEL | MIKROBRIDGE_READ_MASK);
			test_result &= test_value(read_result, 0x00);
			
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_SPI_CS_DEMUX_DEFAULTS | MIKROBRIDGE_READ_MASK);
			test_result &= test_value(read_result, 0xFF);
			
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_UART_TX_DEMUX_DEFAULTS | MIKROBRIDGE_READ_MASK);
			test_result &= test_value(read_result, 0xFF);
			
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_INT_DEMUX_DEFAULTS | MIKROBRIDGE_READ_MASK);
			test_result &= test_value(read_result, 0x00);
			
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_PWM_DEMUX_DEFAULTS | MIKROBRIDGE_READ_MASK);
			test_result &= test_value(read_result, 0x00);
			
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_RST_DEMUX_DEFAULTS | MIKROBRIDGE_READ_MASK);
			test_result &= test_value(read_result, 0xFF);
			
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_DEMUX_PASS_ALL | MIKROBRIDGE_READ_MASK);
			test_result &= test_value(read_result, 0x08);
			
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_CS_DIR | MIKROBRIDGE_READ_MASK);
			test_result &= test_value(read_result, 0x00);
			
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_INT_DIR | MIKROBRIDGE_READ_MASK);
			test_result &= test_value(read_result, 0x00);
			
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_PWM_DIR | MIKROBRIDGE_READ_MASK);
			test_result &= test_value(read_result, 0x00);
			
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_RST_DIR | MIKROBRIDGE_READ_MASK);
			test_result &= test_value(read_result, 0x00);
			
			// If any of the values above were wrong the test result will become zero
			if(test_result != 0)
			{
				GOOD_LED_set_level(true);
			}
			else 
			{
				GOOD_LED_set_level(false);
			}
			
			break;
		case CONFIG_IO:
			ERR_LED_set_level(false);
			GOOD_LED_set_level(false);
			
			// Set up I/O directions and default values
			I2C_0_write1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_CS_DIR, 0x03);
			I2C_0_write1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_INT_DIR, 0x04);
			I2C_0_write1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_PWM_DIR, 0x03);
			I2C_0_write1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_RST_DIR, 0x03);

			break;
		case CONFIG_IO_RDBK:
			// Reset error notification
			GOOD_LED_set_level(false);
			ERR_LED_set_level(false);
			test_result = 1;
			
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_CS_DIR | MIKROBRIDGE_READ_MASK);
			test_result &= test_value(read_result, 0x03);
			
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_INT_DIR | MIKROBRIDGE_READ_MASK);
			test_result &= test_value(read_result, 0x04);
			
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_PWM_DIR | MIKROBRIDGE_READ_MASK);
			test_result &= test_value(read_result, 0x03);
			
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_RST_DIR | MIKROBRIDGE_READ_MASK);
			test_result &= test_value(read_result, 0x03);
			
			if(test_result != 0)
			{
				GOOD_LED_set_level(true);
			}
			else 
			{
				GOOD_LED_set_level(false);
			}
			
			break;
		case DUMMY_SPI0:
			ERR_LED_set_level(false);
			GOOD_LED_set_level(false);
			
			SPI_0_CS_set_level(false);
			SPI_0_write_block("\x57\xB2", 2);
			SPI_0_CS_set_level(true);
			break;
		case I2C_SPI_SWITCH1:
			I2C_0_write1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_SPI_SEL, 0x01);
			break;
		case I2C_SPI_RDBK1:
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_SPI_SEL | MIKROBRIDGE_READ_MASK);
			
			test_value(read_result, 0x01);
			break;
		case DUMMY_SPI1:
			GOOD_LED_set_level(false);
			ERR_LED_set_level(false);
			
			SPI_0_CS_set_level(false);
			SPI_0_write_block("\xAC", 1);
			SPI_0_CS_set_level(true);
			break;
		case I2C_SPI_SWITCH2:
			I2C_0_write1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_SPI_SEL, 0x02);
			break;
		case I2C_SPI_RDBK2:
			read_result = I2C_0_read1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_SPI_SEL | MIKROBRIDGE_READ_MASK);
			
			test_value(read_result, 0x02);
			break;
		case DUMMY_UART0:
			GOOD_LED_set_level(false);
			ERR_LED_set_level(false);
			
			USART_0_write(0x3F);
			USART_0_write(0x49);
			break;
		case SPI_UART_SWITCH1:
			spi_buffer[0] = MIKROBRIDGE_UART_SEL;
			spi_buffer[1] = 0x01;
			
			SPI_0_CS_set_level(false);
			SPI_0_write_block(spi_buffer, 2);
			SPI_0_CS_set_level(true);
			break;
		case SPI_UART_RDBK1:
			spi_buffer[0] = MIKROBRIDGE_UART_SEL | MIKROBRIDGE_READ_MASK;
			spi_buffer[1] = 0x00;
			
			SPI_0_CS_set_level(false);
			SPI_0_exchange_block(spi_buffer, 2);
			SPI_0_CS_set_level(true);
			
			test_value(spi_buffer[1], 0x01);
			break;
		case DUMMY_UART1:
			GOOD_LED_set_level(false);
			ERR_LED_set_level(false);
			
			USART_0_write(0xF3);
			break;
		case SPI_UART_SWITCH2:
			spi_buffer[0] = MIKROBRIDGE_UART_SEL;
			spi_buffer[1] = 0x02;
			
			SPI_0_CS_set_level(false);
			SPI_0_write_block(spi_buffer, 2);
			SPI_0_CS_set_level(true);
			break;
		case SPI_UART_RDBK2:
			spi_buffer[0] = MIKROBRIDGE_UART_SEL | MIKROBRIDGE_READ_MASK;
			spi_buffer[1] = 0x00;
			
			SPI_0_CS_set_level(false);
			SPI_0_exchange_block(spi_buffer, 2);
			SPI_0_CS_set_level(true);
			
			test_value(spi_buffer[1], 0x02);
			break;
		case DUMMY_PWM0:
			GOOD_LED_set_level(false);
			ERR_LED_set_level(false);
			
			// Re-enable the constant PWM for the rest of the tests
			TCA0.SPLIT.CTRLA |= (1 << TCA_SPLIT_ENABLE_bp);
			break;
		case UART_PWM_SWITCH1:
			USART_0_write(MIKROBRIDGE_PWM_SEL);
			USART_0_write(0x01);
			break;
		case UART_PWM_RDBK1:
			USART_0_write(MIKROBRIDGE_PWM_SEL | MIKROBRIDGE_READ_MASK);
			read_result = USART_0_read();

			test_value(read_result, 0x01);
			break;
		case DUMMY_PWM1:
			GOOD_LED_set_level(false);
			ERR_LED_set_level(false);
			break;
		case DUMMY_INT0:
			// Manually input some interrupt signal
			break;
		case I2C_INT_SWITCH1:
			I2C_0_write1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_INT_SEL, 0x01);
			break;
		case DUMMY_INT1:
			// Manually input some interrupt signal
			break;
		case DUMMY_RST0:
			// Active low reset
			RST_set_level(false);
			break;
		case I2C_RST_SWITCH1:
			I2C_0_write1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_RST_SEL, 0x01);
			break;
		case DUMMY_RST1:
			break;
		case I2C_RST_SWITCH_ALL:
			I2C_0_write1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_RST_SEL, 0xFF);
			break;
		case DUMMY_RST_ALL:
			break;
		case I2C_AN_SWITCH1:
			I2C_0_write1ByteRegister(MIKROBRIDGE_I2C_ADDR, MIKROBRIDGE_AN_SEL, 0x02);
			break;
		case DUMMY_I2C:
			I2C_0_writeNBytes(0x2C, "\x3C", 1);
			I2C_0_writeNBytes(0x2C, "\x2A", 1);
			break;
		default:
			// Some error notification
			ERR_LED_set_level(true);
			break;
	}
	
	// Short delay to keep from getting out of control
	_delay_ms(1);
}

uint8_t test_value(uint8_t result, uint8_t test)
{
	if(result == test)
	{
		GOOD_LED_set_level(true);
		ERR_LED_set_level(false);
		return 1;
	}
	else
	{
		GOOD_LED_set_level(false);
		ERR_LED_set_level(true);
		return 0;
	}
}